@@ -11,7 +11,7 @@ sub new {
     my $self = fields::new($class);
     $self->{flow} = [
 	# src, dst, sport, dport, state, sn
-	# state = 0bFfSs: got[F]inack|send[f]in|got[S]ynack|send[s]yn
+	# state = 0bFfSs: send[F]inack|send[f]in|send[S]ynack|send[s]yn
 	# sn gets initialized on sending SYN
 	[ $src,$dst,$sport,$dport,0,     undef ],
 	[ $dst,$src,$dport,$sport,0,     undef ],
@@ -25,6 +25,19 @@ sub write_with_flags {
     my ($self,$dir,$data,$flags,$timestamp) = @_;
     $flags ||= {};
     my $flow = $self->{flow}[$dir];
+
+    if ($flags->{syn} and ($flow->[4] & 0b0001) == 0) {
+	$flow->[4] |= 0b0001;
+	$flow->[5] ||= rand(2**32);
+    }
+    if ($flags->{fin}) {
+	$flow->[4] |= 0b0100;
+    }
+    if ($flags->{ack}) {
+	$flow->[4] |= 0b0010 if ($flow->[4] & 0b0011) == 0b0001; # ACK for SYN
+	$flow->[4] |= 0b1000 if ($flow->[4] & 0b1100) == 0b0100; # ACK for FIN
+    }
+
     my $sn = $flow->[5];
     my $ack = $self->{flow}[$dir?0:1][5];
     $flags->{ack} = 1 if defined $ack;
@@ -75,37 +88,49 @@ sub write {
 
 sub _connect {
     my ($self,$timestamp) = @_;
-    if ( not $self->{flow}[0][4] & 0b01 ) {
-	$self->{flow}[0][5] ||= rand(2**32);
-	$self->write_with_flags(0,'',{ syn => 1 },$timestamp);
-	$self->{flow}[0][4] |= 0b01;
-    }
-    if ( not $self->{flow}[1][4] & 0b01 ) {
-	$self->{flow}[1][5] ||= rand(2**32);
-	$self->write_with_flags(1,'',{ syn => 1, ack => 1 },$timestamp);
-	$self->{flow}[0][4] |= 0b10;
-	$self->{flow}[1][4] |= 0b01;
-    }
-    if ( not $self->{flow}[1][4] & 0b10 ) {
-	$self->ack(0,$timestamp);
-	$self->{flow}[1][4] |= 0b10;
-    }
+    return if ($self->{flow}[0][4] & 0b11) == 0b11
+	&& ($self->{flow}[1][4] & 0b11) == 0b11;
+
+    # client: SYN
+    $self->write_with_flags(0,'',{ syn => 1 },$timestamp) 
+	if ($self->{flow}[0][4] & 0b01) == 0;
+
+    # server: SYN+ACK
+    $self->write_with_flags(1,'',{ 
+	($self->{flow}[1][4] & 0b01) == 0 ? ( syn => 1 ):(),
+	($self->{flow}[1][4] & 0b10) == 0 ? ( ack => 1 ):(),
+    },$timestamp) if ($self->{flow}[1][4] & 0b11) == 0;
+
+    # client: ACK
+    $self->write_with_flags(0,'',{ ack => 1 },$timestamp) 
+	if ($self->{flow}[0][4] & 0b10) == 0;
 }
 
 sub _close {
-    my ($self,$dir,$timestamp) = @_;
-    $self->shutdown($dir||0,$timestamp);
-    $self->shutdown($dir?0:1,$timestamp);
+    my ($self,$timestamp) = @_;
+    $self->_connect($timestamp);
+
+    # client: FIN
+    $self->write_with_flags(0,'',{ fin => 1 },$timestamp) 
+	if ($self->{flow}[0][4] & 0b0100) == 0;
+
+    # server: FIN+ACK
+    $self->write_with_flags(1,'',{ 
+	($self->{flow}[1][4] & 0b0100) == 0 ? ( fin => 1 ):(),
+	($self->{flow}[1][4] & 0b1000) == 0 ? ( ack => 1 ):(),
+    },$timestamp) if ($self->{flow}[1][4] & 0b1100) == 0;
+
+    # client: ACK
+    $self->write_with_flags(0,'',{ ack => 1 },$timestamp) 
+	if ($self->{flow}[0][4] & 0b1000) == 0;
 }
 
 sub shutdown {
     my ($self,$dir,$timestamp) = @_;
-    if ( not $self->{flow}[$dir][4] & 0b0100 ) {
+    if (($self->{flow}[$dir][4] & 0b0100) == 0) {
 	$self->_connect($timestamp);
 	$self->write_with_flags($dir,'',{ fin => 1 },$timestamp);
-	$self->ack($dir?0:1,$timestamp);
-	$self->{flow}[$dir][4] |= 0b0100;
-	$self->{flow}[$dir][4] |= 0b1000;
+	$self->write_with_flags($dir ? 0:1,'',{ ack => 1 },$timestamp);
     }
 }
 
@@ -116,8 +141,7 @@ sub ack {
 
 sub DESTROY {
     my $self = shift;
-    $self->_connect(undef,$self->{last_timestamp});
-    $self->_close(undef,$self->{last_timestamp});
+    $self->_close($self->{last_timestamp});
 }
 
 
